#!BPY

"""
Name: 'J3Dexporter (.j3d)'
Blender: 237
Group: 'Export'
Tooltip: 'Exporter for the Julianna game engine'
"""
# * Must be in object mode when exporting
# * The object you want to export must be selected
# * Make Sure to select the start and end frame of your object. If the endframe is greater than 1 it will be treated as an animation.
#   The default ending frame in blender is 250, so if this is left, this script will assume the endframe is 1 and treat it such.
#	Make sure your start frame is allways 1
#   To do this, select 1-Animation from the drop down box next to the Help menu and select the endframe there.
# * When exporting subsurfd meshes it will export quads, as that is what the subsurf mesh seems to be, no matter should still be 
#   able to import wuads into my engine
# * Try to name the object as the file would be named withouth the j3d extension
# Doesn't export deformed mesh in 2.4, works in 2.37a
import Blender

def write(filename):
		out = file(filename, "w")
		### Check number of frames, if greater than 1 it will be an animation
		startframe = Blender.Get('staframe')
		endframe = Blender.Get('endframe')
		if endframe == 250: endframe = 1
		if endframe > 1 : objecttype = 'Animation'
		else : objecttype = 'Static'
       ### Print the number of faces and number of vertices 
#		for object in Blender.Object.GetSelected(): #You can do just a Get then discard or use anything
		object = Blender.Object.GetSelected()
		objname = object[0].name
		sphere = Blender.NMesh.GetRawFromObject(objname)
		#sphere = object.getData()
		obj = Blender.Object.Get(objname)
		out.write("%d" % startframe + " %d" % endframe + " " + obj.getName() + "\n")    
		#query the object and if a mesh, get the details
		framecount = 1
		while framecount <= endframe:
			object = Blender.Object.GetSelected()
			#sphere = Blender.NMesh.GetRaw(objname) # for actual mesh
			sphere = Blender.NMesh.GetRawFromObject(objname) #for subsurfed mesh
			#sphere.makeDisplayList()
			#sphere = obj.getData()
			i = 0
			out.write("\nVertices " + "%d" % framecount + "\n") ## Write out the current frame locations vertices
			out.write("%d\n" % len(sphere.verts))
			for x in sphere.verts:
				if i > 0: out.write("\n")
				a = sphere.verts[i]
				i += 1
				for n in a.co:
					#out.write(repr(n) + " ")
					out.write("%f " % n)
				for n in a.no:
					out.write("%f " % n)
			i = 0
			out.write("\nFace vertice index and Normals\n")
			out.write("%d\n" % len(sphere.faces))
			for x in sphere.faces:
				if i > 0: out.write("\n")
				a = sphere.faces[i]
				i += 1
				k = 0
				out.write("%d " % len(a.v))
				for j in a.v:
					out.write("%d " % a.v[k].index) 
					k += 1
				for n in a.normal:
					#out.write(repr(n) + " ")
					out.write("%f " % n)
				out.write("%s " % a.image)
				out.write("%s " % a.uv)
			framecount += 1		
			if framecount <= endframe: Blender.Set('curframe',framecount) ## Set Blender to the next frame
			
		Blender.Set('curframe', 1)
				

Blender.Window.FileSelector(write, "Export")
