/* FFilename	game.h
 * date created	19/01/2004
 * by Arek Stankiewicz
 *
 * Description
 * Bangon header file
 *
Copyright Arek Stankiewicz 2004,2005
 This file is part of Hugooski3D.

Hugooski3D is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
Hugooski3D is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with Hugooski3D; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA*/

#ifdef windows
	#include <windows.h>
	#include <GL/glaux.h>
#endif

#include <SDL_image.h>
#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//add all globals, defined, function prototypes and typedefs here

int gameloop( void ); //main game loop
void h3dexit( int code ); //exit code
void setup_opengl (int width, int height); //setup opengl construct

//rendering functions
int loadTextures ( char textureFilename[], int arraySize );
void draw_screen( int* zzmove ); //draw the screen

// input functions 
int*  processkeyboard_events( void );	//as the name suggests
int handle_mousemotion( SDL_MouseMotionEvent *motion, int wtd, int *zzmove );	//as name suggests  wtd is what to do
int handle_key_down( SDL_keysym* keysym, int ownkeys ); //as name suggests
void drawtext( char stext[], int stextsize,  GLfloat offsetx, GLfloat offsety );

void drawScene( void );
int* drawScene2( GLfloat cameraPosX, GLfloat cameraPosY, GLfloat cameraPosZ, int rotateChar, int DoRotateChar, GLfloat yrot, int *cameray );
int detectCollision( int *model1, int *model2 );
void terrainDetection( int *character, int *land, int *cameray);

void calculateExtremities(int *modeltoCalculate);
//void loadModel (int *modelTL, int wtl);
int loadModel ( int *modelTL, char filena[] );
char cleanLoadString ( char toChange );
#ifdef windows
	#define SDL_main main
#endif

#define INIT	0
#define INTRO	1
#define MENU	2
#define GAME	3
#define SHUTDOWN	4
#define PI 3.141592654

#define NUM_FACES 600 //How many faces in the model
#define NUM_FACE_VERTICES (NUM_FACES*3) 

#define CAMERA_TYPE 3 	//1 = Top view follow
						//2 = ??
						//3 = 3rd person

#define sq(x) ((x)*(x))

typedef struct {
    int strafeRight;
    int strafeLeft;
    int forward;
    int backward;
    int lookLeft;
    int lookRight;
    int lookup;
    int lookdown;
	int cameraUp;
	int cameraDown;
    GLfloat rspeed; //so we can modify the look sped depending on how fast the mouse is moved
    GLfloat yspeed; //different speed for looking up down also
} cmMov;

typedef struct {
    SDLKey sleft;
    SDLKey sright;
    SDLKey frwrd;
    SDLKey bckwrd;
	SDLKey camup;
	SDLKey camdwn;
} keymap;

typedef struct {  // A lot of this stuff should be loaded in the file loader
    GLfloat 		colors [3*NUM_FACE_VERTICES];
    long int 		vertices; //Holds how many vertices
    unsigned short 	indices [NUM_FACE_VERTICES];
    GLfloat 		normals [3 * NUM_FACE_VERTICES]; //the vertice normals for smooth shading
    GLfloat 		face_normals [3 * NUM_FACE_VERTICES]; //the face normals for facetted shading
    GLfloat 		modelVertices[3*NUM_FACE_VERTICES];// this array holds all the vertices that draw the sphere
    GLfloat 		modelTexcoords[2*NUM_FACE_VERTICES];
    int			textureIndex; //the index of the texture to use on the model 
    int			textureLoaded; //for now we will use only one texture per model
    GLfloat 		extremx[2]; // The box extremeties for the model for crude collision detection
    GLfloat 		extremy[2];
    GLfloat 		extremz[2];
    GLfloat 		radius;
    int			numVerts;
    char		type[8]; //can be static (like house) anim(like tree) animmove(like character)
    char		filename[30]; //for debugging, the filename model is loaded from
    int 		lastFrame; //If 1 the model must not be animated
    int 		currframe; //the current animation frame
    Uint32 		frameTime; //lets us know when to pop into the next frame or not
    char		name[20]; //the name of the model
    GLfloat 		pos[3]; //x y and z position of the centre of the model, set in calculateExtremeties
    GLfloat		prevpos[3];//previous pos to calculate the direction vector 
    GLfloat		rot[3]; //rotate by how many degrees on each angle, he initial value in world.jen
    int			hit; //if touched with hero
	//add the rotation position, rotx roty rotb, initial values in the world.jen file as they may appear upside down etc
	//when exported from blender
} model;

typedef struct {
	
	model frame[40]; //each frame for an animated model

} animmodel;

typedef struct {
		
		int ModelCollided; //set if the model has collided with something and reverted to prev position

} Messages;
